<?php

namespace Drupal\site_assistant\Entity;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Condition\ConditionInterface;
use Drupal\Core\Entity\EditorialContentEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\site_assistant\AssistantInterface;
use Drupal\user\EntityOwnerTrait;

/**
 * Defines the site assistant entity class.
 *
 * @ContentEntityType(
 *   id = "assistant",
 *   label = @Translation("Site assistant"),
 *   label_collection = @Translation("Site assistants"),
 *   label_singular = @Translation("site assistant"),
 *   label_plural = @Translation("site assistants"),
 *   label_count = @PluralTranslation(
 *     singular = "@count site assistants",
 *     plural = "@count site assistants",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\site_assistant\AssistantListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "form" = {
 *       "add" = "Drupal\site_assistant\Form\AssistantForm",
 *       "edit" = "Drupal\site_assistant\Form\AssistantForm",
 *       "delete" = "Drupal\site_assistant\Form\AssistantDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "assistant",
 *   data_table = "assistant_field_data",
 *   revision_table = "assistant_revision",
 *   revision_data_table = "assistant_field_revision",
 *   show_revision_ui = TRUE,
 *   translatable = TRUE,
 *   admin_permission = "add assistant content",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "vid",
 *     "langcode" = "langcode",
 *     "label" = "title",
 *     "status" = "status",
 *     "published" = "status",
 *     "uuid" = "uuid",
 *     "owner" = "uid",
 *   },
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_uid",
 *     "revision_created" = "revision_timestamp",
 *     "revision_log_message" = "revision_log",
 *   },
 *   links = {
 *     "collection" = "/admin/content/site_assistant/assistant/list",
 *     "add-form" = "/admin/content/site_assistant/assistant/add",
 *     "canonical" = "/admin/content/site_assistant/assistant/{assistant}",
 *     "edit-form" = "/admin/content/site_assistant/assistant/{assistant}/edit",
 *     "delete-form" = "/admin/content/site_assistant/assistant/{assistant}/delete",
 *   },
 * )
 */
class Assistant extends EditorialContentEntityBase implements AssistantInterface {

  use EntityOwnerTrait;

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage): void {
    parent::preSave($storage);
    if (!$this->getOwnerId()) {
      // If no owner has been set explicitly, make the anonymous user the owner.
      $this->setOwnerId(0);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {
    $fields = parent::baseFieldDefinitions($entity_type);
    $fields += static::ownerBaseFieldDefinitions($entity_type);

    $fields['title'] = BaseFieldDefinition::create('string')
      ->setRevisionable(TRUE)
      ->setTranslatable(TRUE)
      ->setLabel(t('Title'))
      ->setRequired(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['status']
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => TRUE,
        ],
        'weight' => 120,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['uid']
      ->setLabel(t('Authored by'))
      ->setDescription(t('The username of the content author.'))
      ->setRevisionable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Authored on'))
      ->setDescription(t('The time that the node was created.'))
      ->setRevisionable(TRUE)
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'timestamp',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 10,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setTranslatable(TRUE)
      ->setDescription(t('The time that the site assistant was last edited.'));

    $fields['weight'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Weight'))
      ->setDescription(t('The weight of this term in relation to other terms.'))
      ->setDefaultValue(0);

    $fields['content'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Content'))
      ->setDescription(t('The content of the Site assistant.'))
      ->setTranslatable(TRUE)
      ->setRevisionable(TRUE)
      ->setRequired(TRUE)
      ->setSetting('target_type', 'assistant_list_entry')
      ->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'entity_reference_entity_view',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'inline_entity_form_complex',
        'weight' => -1,
        'region' => 'content',
        'settings' => [
          'form_mode' => 'default',
          'revision' => TRUE,
          'override_labels' => TRUE,
          'label_singular' => t('Entry'),
          'label_plural' => t('Entries'),
          'collapsible' => FALSE,
          'collapsed' => FALSE,
          'allow_asymmetric_translation' => TRUE,
        ],
      ]);


    $fields['visibility_plugins'] = BaseFieldDefinition::create('plugin:condition')
      ->setLabel(t('Visibility conditions'))
      ->setDescription(t('Field to save the visibility condition plugins.'))
      ->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED)
      ->setDisplayOptions('form', [
        'type' => 'plugin_selector:plugin_select_list',
        'weight' => 0,
      ]);

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getWeight(): int {
    return $this->get('weight')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setWeight(int $weight): static {
    $this->set('weight', $weight);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts(): array {
    $contexts = parent::getCacheContexts();

    foreach ($this->getVisibilityPlugins() as $plugin) {
      $contexts = Cache::mergeContexts($contexts, $plugin->getCacheContexts());
    }

    return $contexts;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    $tags = parent::getCacheTags();

    foreach ($this->getVisibilityPlugins() as $plugin) {
      $tags = Cache::mergeTags($tags, $plugin->getCacheTags());
    }

    return $tags;
  }

  /**
   * {@inheritdoc}
   */
  public function getVisibilityPlugins(): ?iterable {
    $plugins = [];

    if (!$this->visibility_plugins->isEmpty()) {
      /** @var \Drupal\plugin\Plugin\Field\FieldType\PluginCollectionItem $plugin_collection_item */
      foreach ($this->visibility_plugins as $plugin_collection_item) {
        /** @var \Drupal\Core\Condition\ConditionInterface $plugin */
        if ($plugin = $plugin_collection_item->getContainedPluginInstance()) {
          if ($plugin instanceof ConditionInterface) {
            $plugins[] = $plugin;
          }
        }
      }
    }
    return $plugins;
  }

}
